%{

#ifndef __G_LIB_H__
#include <glib.h>
#endif

#include "rt-compile.h"
#include "rt-global.h"
#include "ct-compile.h"

GList* (*mk_bytecmp) (int ftype, int rel_opcode, guint8 *bytes);
void (*mk_optimize) (GList *L);

/* The encapsulation type for which we are compiling the filter */
int comp_encap_type;
int filter_parsed = 0;

bytecmp_info *bytecmp;
eitherof_info *either;
GList *L1, *L2;

bytecmp_info bytecmp_table[] = {
	{ ETH_ADDR, 3, 0, 6 },
	{ TR_SRCVENDOR, 2, 8, 3 },
	{ ETH_VENDOR, 3, 0, 3 },
	{ TR_VENDOR, 3, 0, 3 },
	{ ETH_SRC, 2, 6, 6 },
	{ ETH_SRCVENDOR, 2, 6, 3 },
	{ TR_DSTVENDOR, 2, 2, 3 },
	{ ETH_DST, 2, 0, 6 },
	{ TR_DST, 2, 2, 6 },
	{ TR_ADDR, 3, 0, 6 },
	{ TR_SRC, 2, 8, 6 },
	{ ETH_DSTVENDOR, 2, 0, 3 },
	{ 0, 0, 0, 0 }
};
/*bytecmp_info bytecmp_table[] = {
	{ ETH_TYPE, 12, 2 },
	{ TR_DST, 2, 6 },
	{ TR_SRC, 8, 6 },
	{ ETH_DSTVENDOR, 0, 3 },
	{ 0, 0, 0 }
};*/

eitherof_info eitherof_table[] = {
	{ ETH_ADDR, 3, ETH_SRC, ETH_DST },
	{ ETH_VENDOR, 3, ETH_SRCVENDOR, ETH_DSTVENDOR },
	{ TR_VENDOR, 3, TR_SRCVENDOR, TR_DSTVENDOR },
	{ TR_ADDR, 3, TR_SRC, TR_DST },
	{ 0, 0, 0, 0 }
};
/*eitherof_table[] = {
	{ TR_VENDOR, CTYPE_BYTECMP, TR_SRCVENDOR, TR_DSTVENDOR },
	{ TR_ADDR, CTYPE_BYTECMP, TR_SRCADDR, TR_DSTADDR }
};
*/
%}

%union {
	gint	d;
	guint8	*b;
	GString	*s;
	GList	*L;
}

%type <d>	bytecmp_lval
%type <L>	sentence bytecmp_relation
%type <d>	bytecmp_test;

%token <b>	BYTES
%token <s>	QUOTED TEXT
%token <d>	NUMBER
%token <d>	EQ NE

%token <d>	ETH_ADDR
%token <d>	TR_SRCVENDOR
%token <d>	ETH_VENDOR
%token <d>	TR__NEXT
%token <d>	TR_VENDOR
%token <d>	ETH_SRC
%token <d>	ETH_SRCVENDOR
%token <d>	TR_SR
%token <d>	TR_DSTVENDOR
%token <d>	ETH_DST
%token <d>	TR_DST
%token <d>	TR_ADDR
%token <d>	TR_SRC
%token <d>	ETH_DSTVENDOR

%%

paragraph: /* EMPTY */
		| paragraph sentence { mk_optimize($2); filter_parsed = 1; }
		;

sentence:	bytecmp_relation { $$ = $1 }
		;
	

bytecmp_relation:	bytecmp_lval bytecmp_test BYTES
{
	bytecmp = lookup_bytecmp($1);
	if (bytecmp->ctype == CTYPE_EITHEROF) {
		either = lookup_eitherof($1);
		L1 = mk_bytecmp(either->field1, $2, $3);
		L2 = mk_bytecmp(either->field2, $2, $3);
		$$ = g_list_concat(L1, L2);
	}
	else {
		$$ = mk_bytecmp($1, $2, $3);
	}
}
		;

bytecmp_lval:		ETH_ADDR { $$ = ETH_ADDR; }
	|		TR_SRCVENDOR { $$ = TR_SRCVENDOR; }
	|		ETH_VENDOR { $$ = ETH_VENDOR; }
	|		TR_VENDOR { $$ = TR_VENDOR; }
	|		ETH_SRC { $$ = ETH_SRC; }
	|		ETH_SRCVENDOR { $$ = ETH_SRCVENDOR; }
	|		TR_DSTVENDOR { $$ = TR_DSTVENDOR; }
	|		ETH_DST { $$ = ETH_DST; }
	|		TR_DST { $$ = TR_DST; }
	|		TR_ADDR { $$ = TR_ADDR; }
	|		TR_SRC { $$ = TR_SRC; }
	|		ETH_DSTVENDOR { $$ = ETH_DSTVENDOR; }
	;
/*bytecmp_lval:	TR_DST { $$ = TR_DST; }
	|	TR_SRC { $$ = TR_SRC; }
	|	TR_SRCVENDOR { $$ = TR_SRCVENDOR; }
	|	TR_DSTVENDOR { $$ = TR_DSTVENDOR; }
	;*/

bytecmp_test:	EQ { $$ = EQ; }
	|	NE { $$ = NE; }
	;

%%

bytecmp_info*
lookup_bytecmp(int ftype)
{
	bytecmp_info *b = &bytecmp_table[0];
	bytecmp_info *ret_val = NULL;

	/* find the field in the table */
	while (b->ftype != 0) {
		if (b->ftype == ftype) {
			ret_val = b;
			break;
		}
		else {
			b++;
		}
	}

	return ret_val;
}


eitherof_info*
lookup_eitherof(int ftype)
{
	eitherof_info *e = &eitherof_table[0];
	eitherof_info *ret_val = NULL;

	/* find the field in the table */
	while (e->ftype != 0) {
		if (e->ftype == ftype) {
			ret_val = e;
			break;
		}
		else {
			e++;
		}
	}

	return ret_val;
}
